from __future__ import division
import numpy as np
from scipy.io.wavfile import read, write
from .logmmse import logmmse as _logmmse
from .utils import to_float, from_float

np.seterr('raise')

def mono_logmmse(m_input, fs, dtype, initial_noise=6, window_size=0, noise_threshold=0.15):
    num_frames = len(m_input)
    chunk_size = int(np.floor(60*fs))
    m_output = np.array([], dtype=dtype)
    saved_params = None
    frames_read = 0
    while frames_read < num_frames:
        frames = num_frames - frames_read if frames_read + chunk_size > num_frames else chunk_size
        signal = m_input[frames_read:frames_read + frames]
        frames_read = frames_read + frames
        _output, saved_params = _logmmse(signal, fs, initial_noise, window_size, noise_threshold, saved_params)
        m_output = np.concatenate((m_output, from_float(_output, dtype)))
    return m_output

def logmmse(data, sampling_rate, output_file=None, initial_noise=6,
            window_size=0, noise_threshold=0.15):
    data, dtype = to_float(data)
    data += np.finfo(np.float64).eps
    if data.ndim == 1:
        output = mono_logmmse(data, sampling_rate, dtype, initial_noise, window_size,
                              noise_threshold)
    else:
        output = []
        for _, m_input in enumerate(data.T):
            output.append(mono_logmmse(m_input, sampling_rate, dtype, initial_noise,
                                       window_size, noise_threshold))
    output = np.array(output)
    if output_file is not None:
        write(output_file, sampling_rate, output.T)
    return output.T

def logmmse_from_file(input_file, output_file=None, initial_noise=6,
                      window_size=0, noise_threshold=0.15):
    sampling_rate, data = read(input_file, 'r')
    return logmmse(data, sampling_rate, output_file, initial_noise,
                   window_size, noise_threshold)



from setuptools import setup
with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='logmmse',
    version='1.4',
    description='A python implementation of the LogMMSE speech enhancement/noise reduction alogrithm',
    long_description=long_description,
    long_description_content_type="text/markdown",
    license='MIT',
    url='https://github.com/wilsonchingg/logmmse',
    packages=['logmmse'],
    author='Rajiv Poddar, Wilson Ching (Fork Author)',
    author_email='wilsonchingg96@gmail.com',
    install_requires=[
        'numpy',
        'scipy',
    ],
)



from __future__ import division
import numpy as np

def to_float(_input):
    if _input.dtype == np.float64:
        return _input, _input.dtype
    elif _input.dtype == np.float32:
        return _input.astype(np.float64), _input.dtype
    elif _input.dtype == np.uint8:
        return (_input - 128) / 128., _input.dtype
    elif _input.dtype == np.int16:
        return _input / 32768., _input.dtype
    elif _input.dtype == np.int32:
        return _input / 2147483648., _input.dtype
    raise ValueError('Unsupported wave file format, please contact the author')

def from_float(_input, dtype):
    if dtype == np.float64:
        return _input, np.float64
    elif dtype == np.float32:
        return _input.astype(np.float32)
    elif dtype == np.uint8:
        return ((_input * 128) + 128).astype(np.uint8)
    elif dtype == np.int16:
        return (_input * 32768).astype(np.int16)
    elif dtype == np.int32:
        print(_input)
        return (_input * 2147483648).astype(np.int32)
    raise ValueError('Unsupported wave file format, please contact the author')



from __future__ import division
import math
import numpy as np
from scipy.special import expn

def logmmse(x, Srate, noise_frames=6, Slen=0, eta=0.15, saved_params=None):
    if Slen == 0:
        Slen = int(math.floor(0.02 * Srate))

    if Slen % 2 == 1:
        Slen = Slen + 1

    PERC = 50
    len1 = int(math.floor(Slen * PERC / 100))
    len2 = int(Slen - len1)

    win = np.hanning(Slen)
    win = win * len2 / np.sum(win)
    nFFT = 2 * Slen

    x_old = np.zeros(len1)
    Xk_prev = np.zeros(len1)
    Nframes = int(math.floor(len(x) / len2) - math.floor(Slen / len2))
    xfinal = np.zeros(Nframes * len2)

    if saved_params is None:
        noise_mean = np.zeros(nFFT)
        for j in range(0, Slen*noise_frames, Slen):
            noise_mean = noise_mean + np.absolute(np.fft.fft(win * x[j:j + Slen], nFFT, axis=0))
        noise_mu2 = (noise_mean / noise_frames) ** 2
    else:
        noise_mu2 = saved_params['noise_mu2']
        Xk_prev = saved_params['Xk_prev']
        x_old = saved_params['x_old']

    aa = 0.98
    mu = 0.98
    eta = 0.15
    ksi_min = 10 ** (-25 / 10)

    for k in range(0, Nframes*len2, len2):
        insign = win * x[k:k + Slen]

        spec = np.fft.fft(insign, nFFT, axis=0)
        sig = np.absolute(spec)
        sig2 = sig ** 2

        gammak = np.minimum(sig2 / noise_mu2, 40)

        if Xk_prev.all() == 0:
            ksi = aa + (1 - aa) * np.maximum(gammak - 1, 0)
        else:
            ksi = aa * Xk_prev / noise_mu2 + (1 - aa) * np.maximum(gammak - 1, 0)
            ksi = np.maximum(ksi_min, ksi)

        log_sigma_k = gammak * ksi/(1 + ksi) - np.log(1 + ksi)
        vad_decision = np.sum(log_sigma_k)/Slen
        if vad_decision < eta:
            noise_mu2 = mu * noise_mu2 + (1 - mu) * sig2

        A = ksi / (1 + ksi)
        vk = A * gammak
        ei_vk = 0.5 * expn(1, vk)
        hw = A * np.exp(ei_vk)
        sig = sig * hw
        Xk_prev = sig ** 2
        xi_w = np.fft.ifft(hw * spec, nFFT, axis=0)
        xi_w = np.real(xi_w)

        xfinal[k:k + len2] = x_old + xi_w[0:len1]
        x_old = xi_w[len1:Slen]

    return xfinal, {'noise_mu2': noise_mu2, 'Xk_prev': Xk_prev, 'x_old': x_old}
